﻿using System;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Linq;
using System.Reflection;

namespace GailTesterAssembly
{
    // Enum example to use in TD tester
    public enum Season
    {
        Spring,
        Summer,
        Autumn,
        Winter
    }

    public enum Size : ushort
    {
        None = 0,
        Unknown = 1,
        ConnectionLost = 100,
        OutlierReading = 200
    }

    [Flags]
    public enum Days
    {
        None = 0b_0000_0000,  // 0
        Monday = 0b_0000_0001,  // 1
        Tuesday = 0b_0000_0010,  // 2
        Wednesday = 0b_0000_0100,  // 4
        Thursday = 0b_0000_1000,  // 8
        Friday = 0b_0001_0000,  // 16
        Saturday = 0b_0010_0000,  // 32
        Sunday = 0b_0100_0000,  // 64
        Weekend = Saturday | Sunday
    }

    public class TestClass
    {
        public string mystring { get; set; }
        public int mynumber { get; set; }

        public TestClass()
        {
            Debug.WriteLine("[GailTesterAssembly] CONSTRUCTOR TestClass()");
        }

        ~TestClass()
        {
            Debug.WriteLine("[GailTesterAssembly] DESTRUCTOR TestClass()");
        }
    }

    // Holds all basic methods to pass and return data
    public class TestMethods
    {
        private TestClass myobject;     // Holds data to be compared

        public TestMethods()
        {
            Debug.WriteLine("[GailTesterAssembly] CONSTRUCTOR TestMethods() from " + Assembly.GetExecutingAssembly().Location);

        }

        ~TestMethods()
        {
            Debug.WriteLine("[GailTesterAssembly] DESTRUCTOR ~TestMethods()");
        }

        public int Pass_NumberInt(int input, out int output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return input;
        }

        public uint Pass_NumberUInt(uint input, out uint output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return output;
        }

        public double Pass_NumberDouble(double input, out double output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return input;
        }

        public byte Pass_NumberByte(byte input, out byte output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return input;
        }

        public decimal Pass_NumberDecimal(decimal input, out decimal output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return input;
        }

        public float Pass_NumberFloat(float input, out float output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return input;
        }

        public sbyte Pass_NumberSByte(sbyte input, out sbyte output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return input;
        }

        public long Pass_NumberLong(long input, out long output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return input;
        }

        public ulong Pass_NumberULong(ulong input, out ulong output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return output;
        }

        public short Pass_NumberShort(short input, out short output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return output;
        }

        public ushort Pass_NumberUShort(ushort input, out ushort output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return output;
        }

        public char Pass_NumberChar(char input, out char output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return input;
        }

        public bool Pass_EnumSeason(Season value)
        {
            bool bRet = (value == Season.Winter);
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + bRet);
            return bRet;
        }

        public bool Pass_EnumDay(Days value)
        {
            bool bRet = (value == Days.Monday);
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + bRet);
            return bRet;
        }

        public bool Pass_EnumSize(Size value)
        {
            bool bRet = (value == Size.ConnectionLost);
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + bRet);
            return bRet;
        }

        public bool Pass_IntArray(int[] values)
        {
            int[] checkvalues = { 128, -1, 0, 45000 };
            string debug = string.Join(", ", checkvalues.Select(x => x.ToString()).ToArray());

            bool areEqual = checkvalues.SequenceEqual(values);

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({debug}) -> " + areEqual);

            return areEqual;
        }

        public bool Pass_BinaryData(byte[] values)
        {
            bool areEqual = false;

            string hexvalue = ByteArrayToString(values);

            if (values.Length == 4)
            {
                areEqual = (values[0] == 0 && values[1] == 1 && values[2] == 32 && values[3] == 255);
            }

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}( {hexvalue}, size={values.Length}) -> " + areEqual);

            return areEqual;
        }

        public bool Pass_ObjectArray(TestClass[] values)
        {
            bool areEqual = false;

            if (values.Length == 3)
            {
                if (values[0].mystring == "First element" && values[1].mystring == "Second element" && values[2].mystring == "Third element")
                    if (values[0].mynumber == 1234 && values[1].mynumber == 999 && values[2].mynumber == -10)
                        areEqual = true;
            }

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({values.Length} items) -> " + areEqual);

            return areEqual;
        }

        public bool Pass_BoolTrue(bool value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == true));
            return (value == true);
        }

        public bool Pass_BoolFalse(bool value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == false));
            return (value == false);
        }

        public bool Pass_StringNull(string value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == null));
            return (value == null);
        }

        public bool Pass_StringEmpty(string value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == ""));
            return (value == "");
        }

        public bool Pass_StringSpace(string value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == " "));
            return (value == " ");
        }

        public bool Pass_StringXML(string value)
        {
            string xml = "<note>\r\n<to>Tove</to>\r\n<from>Jani</from>\r\n<heading>Reminder</heading>\r\n<body>Don't forget me this weekend!</body>\r\n</note>";

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == xml));
            return (value == xml);
        }

        public bool Pass_StringTab(string value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == "\t"));
            return (value == "\t");
        }

        public bool Pass_StringNewline(string value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == "\r\n"));
            return (value == "\r\n");
        }

        public bool Pass_StringHelloWorld(string value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == "Hello World!"));
            return (value == "Hello World!");
        }

        public bool Pass_StringArray(string[] cars)
        {
            string[] checkcars = { "Volvo", "BMW", "Ford", "Mazda" };
            string debug = string.Join(", ", cars);

            bool areEqual = checkcars.SequenceEqual(cars);

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({debug}) -> " + areEqual);


            return areEqual;
        }

        public bool Pass_SeasonEnumValue(Season input, out Season output)
        {
            if (input == Season.Autumn)
            {
                output = Season.Winter;
                Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> Return true");
                return true;
            }
            else
            {
                output = Season.Summer;
                Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> Return false");
                return false;
            }
        }

        public bool Pass_DateTimeArray(DateTime[] somedays)
        {
            DateTime[] checkdays = { DateTime.Today, DateTime.Today.AddDays(1) };

            bool areEqual = checkdays.SequenceEqual(somedays);

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({somedays}) -> Return {areEqual}");

            return areEqual;
        }

        public DateTime Pass_DateTime(DateTime input, out DateTime output)
        {
            output = input;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input},{output}) -> {input}");
            return output;
        }

        // Not generated by .NET Explorer. Manually added it gives runtime error
        public bool Pass_BoolNull(bool? value)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value}) -> " + (value == false));
            return (value == null);
        }

        // Not generated by .NET Explorer. Manually added also does not work
        public int? Pass_NumberIntNull(int? input)
        {
            bool areequal = (input == null);
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}( null ) input is null - {areequal} -> " + (input == null));
            return input;
        }

        public byte[] PassImage(byte[] image)
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}(byte[{image.Length}]) -> Return byte[" + image.Length + "]");
            return image;
        }

        public bool Return_BoolTrue()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> true");
            return true;
        }

        public bool Return_BoolFalse()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> false");
            return false;
        }

        public string Return_StringNull()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> null");
            return null;
        }

        public string Return_StringEmpty()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> ''");
            return "";
        }

        public string Return_StringSpace()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> ' '");
            return " ";
        }

        public string Return_NetVersion()
        {
            string version = Environment.Version.ToString();
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> {version}");
            return version;
        }

        public string Return_StringTab()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> \\t");
            return "\t";
        }

        public string Return_StringXML()
        {
            string xml = "<note>\r\n<to>Tove</to>\r\n<from>Jani</from>\r\n<heading>Reminder</heading>\r\n<body>Don't forget me this weekend!</body>\r\n</note>";

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> " + xml);
            return xml;
        }

        public string Return_StringNewline()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> \\r\\n");
            return "\r\n";
        }

        public string Return_StringHelloWorld()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> 'Hello World!'");
            return "Hello World!";
        }

        public DateTime Return_DateTimeToday()
        {
            DateTime today = DateTime.Today;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> " + today);
            return today;
        }

        public DateTime Return_DateTime()
        {
            DateTime MyDateTime = new DateTime(2015, 12, 31, 5, 10, 20);
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> " + MyDateTime);
            return MyDateTime;
        }

        public TestClass Return_TestObject()
        {
            myobject = new TestClass();

            myobject.mystring = "This is a string";
            myobject.mynumber = 12345;

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> " + myobject);
            return myobject;
        }

        public Season Return_SeasonEnumValue()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> Season.Winter");
            return Season.Winter;
        }

        // Not generated by .NET Explorer. But manually added it works
        public bool? Return_BoolNull()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> null");
            return null;
        }

        // Not generated by .NET Explorer. But manually added it works
        public int? Return_IntNull()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> null");
            return null;
        }

        // Not generated by .NET Explorer. But manually added it works
        public DateTime? Return_DateTimeNull()
        {
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> null");
            return null;
        }

        // Not generated by .NET Explorer. But manually added it works
        public double[] Return_NumberDoubleArray()
        {
            double[] somenumbers = { 1, -2.5, 55.59, 1000 };

            string debug = string.Join(", ", somenumbers.Select(x => x.ToString()).ToArray());
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> " + debug);
            return somenumbers;
        }

        // Not generated by .NET Explorer. But manually added it works
        public TestClass[] Return_ObjectArray()
        {
            TestClass[] values = {
                new TestClass{ mystring = "First element", mynumber = 1234 },
                new TestClass{ mystring = "Second element", mynumber = 999 },
                new TestClass{ mystring = "Third element", mynumber = -10 }
                };

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> " + values.ToString());
            return values;
        }

        // Not generated by .NET Explorer. But manually added it works
        public string[] Return_StringArray()
        {
            string[] cars = { "Volvo", "BMW", "Ford", "Mazda" };
            string debug = string.Join(", ", cars);
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> {debug}");
            return cars;
        }

        // Not generated by .NET Explorer. But manually added it works
        public int[] Return_IntArray()
        {
            int[] values = { 128, -1, 0, 45000 };
            string debug = string.Join(", ", values.Select(x => x.ToString()).ToArray());
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> " + debug);
            return values;
        }

        public bool GetTestObject(out TestClass testobject)
        {
            if (myobject == null)
            {
                myobject = new TestClass();

                myobject.mystring = "This is a string";
                myobject.mynumber = 12345;
            }

            testobject = myobject;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}( ) -> true");
            return true;
        }

        public bool SetTestObject(TestClass testobject)
        {
            myobject = testobject;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}( ) -> true");
            return true;
        }

        public bool NumberParams(bool positive, out int pnint, out double pndouble, out float pnfloat, out sbyte pnsbyte, out byte pnbyte, out decimal pndecimal)
        {
            pnint = 1 * (int)(positive ? 1 : -1);
            pndouble = 2.12345 * (double)(positive ? 1 : -1);
            pnfloat = 3.12345F * (float)(positive ? 1 : -1);
            pnsbyte = (sbyte)(4 * (sbyte)(positive ? 1 : -1));
            pnbyte = 5;
            pndecimal = (decimal)(6.12345 * (positive ? 1 : -1));
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({positive}, {pnint}, {pndouble}, {pnfloat}, {pnsbyte}, {pnbyte}, {pndecimal}) -> true");

            return true;
        }

        public string StringInputIsOutput(string value, out string returnStr )
        {
            returnStr = value;
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({value},{returnStr}) -> {value}");
            return value;
        }

        public bool StringArrayInputIsOutput(string[] input, out string[] output)
        {
            output = input;
            string debug1 = string.Join(", ", input);
            string debug2 = string.Join(", ", output);
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({debug1},{debug2}) -> true");
            return true;
        }

        public bool StringArrayInputIsOutput2(string[] input, out string[] output1, out string[] output2)
        {
            output1 = input;
            output2 = input;
            string debug1 = string.Join(", ", input);
            string debug2 = string.Join(", ", output1);
            string debug3 = string.Join(", ", output2);
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({debug1},{debug2},{debug3}) -> true");
            return true;
        }

        public byte[] GetImage()
        {
            byte[] buffer;
            try
            {
                buffer = File.ReadAllBytes(GetAssemblyDirectory() + "\\ForumLogo.bmp");
                Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> Return ForumLogo.bmp byte[{buffer.Length}]");
                return buffer;
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}() -> Error " + ex.ToString());
                return null;
            }
        }

        public bool CheckImage( byte[] image)
        {
            bool areEqual = false;

            try
            {
                byte[] buffer = File.ReadAllBytes(GetAssemblyDirectory() + "\\ForumLogo.bmp");
                areEqual = buffer.SequenceEqual(image);

                Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}(byte[{image.Length}]) check byte[{buffer.Length}] -> Return {areEqual}");
                return areEqual;
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}(byte[{image.Length}]) -> Error " + ex.ToString());
                return false;
            }
        }

        public bool Check_SeasonEnumValue(Season input)
        {
            bool areEqual = (input == Season.Autumn);

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}({input}) -> Return {areEqual}");
            return areEqual;

        }

        public void ForceGarbageCollection()
        {
            // Just a way to force garbage collection
            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}()");
            GC.Collect();
        }

        public void Reset_TestObject()
        {
            myobject = new TestClass();

            myobject.mystring = "This is a string";
            myobject.mynumber = 12345;

            Debug.WriteLine($"[GailTesterAssembly] {System.Reflection.MethodBase.GetCurrentMethod().Name}()");
        }


        //PRIVATE HELPER METHODS >>>>

        private string ByteArrayToString(byte[] ba)
        {
            StringBuilder hex = new StringBuilder(ba.Length * 2);
            foreach (byte b in ba)
                hex.AppendFormat("{0:x2}", b);
            return hex.ToString();
        }

        private string GetAssemblyDirectory( )
        {
            string codeBase = Assembly.GetExecutingAssembly().CodeBase;
            UriBuilder uri = new UriBuilder(codeBase);
            string path = Uri.UnescapeDataString(uri.Path);
            return Path.GetDirectoryName(path);
        }
    }
}
